package com.ruoyi.file.service.impl;

import java.util.List;

import com.ruoyi.common.core.redis.RedisCache;
import com.ruoyi.common.exception.ASSERT;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import com.ruoyi.file.utils.ProjectUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.file.mapper.ProjectMapper;
import com.ruoyi.file.domain.Project;
import com.ruoyi.file.service.IProjectService;
import org.springframework.transaction.annotation.Transactional;
import org.apache.commons.collections4.ListUtils;

import javax.annotation.PostConstruct;

/**
 * 项目Service业务层处理
 * 
 * @author zhao
 * @date 2023-10-19
 */
@Service
public class ProjectServiceImpl implements IProjectService 
{
    @Autowired
    private ProjectMapper projectMapper;

    /**
     * 项目启动时，初始化字典到缓存
     */
    @PostConstruct
    public void init()
    {
        ProjectUtils.ReloadCache(projectMapper);
    }

    /**
     * 查询项目
     * 
     * @param id 项目主键
     * @return 项目
     */
    @Override
    public Project selectProjectById(Long id)
    {
        return projectMapper.selectProjectById(id);
    }

    /**
     * 查询项目列表
     * 
     * @param project 项目
     * @return 项目
     */
    @Override
    public List<Project> selectProjectList(Project project)
    {
        return projectMapper.selectProjectList(project);
    }

    /**
     * 新增项目
     * 
     * @param project 项目
     * @return 结果
     */
    @Override
    public int insertProject(Project project)
    {
        project.setCreateTime(DateUtils.getNowDate());
        int r = projectMapper.insertProject(project);
        ProjectUtils.Set(project);
        return r;
    }

    /**
     * 批量新增项目
     *
     * @param list 项目
     * @return 结果
     */
    @Override
    @Transactional
    public int insertProjectBatch(List<Project> list){
        List<List<Project>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
                projectMapper.insertProjectBatch(splist);
        });
        return 1;
    }

    /**
     * 修改项目
     * 
     * @param project 项目
     * @return 结果
     */
    @Override
    public int updateProject(Project project)
    {
        project.setUpdateTime(DateUtils.getNowDate());
        int r = projectMapper.updateProject(project);
        ProjectUtils.Set(project);
        return r;
    }

    /**
     * 批量修改 项目
     *
     * @param list 项目
     * @return 结果
     */
    @Override
    @Transactional
    public int updateProjectBatch(List<Project> list) {
        List<List<Project>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
                projectMapper.updateProjectBatch(splist);
        });
        return 1;
    }

    /**
     * 批量删除项目
     * 
     * @param ids 需要删除的项目主键
     * @return 结果
     */
    @Override
    public int deleteProjectByIds(Long[] ids)
    {
        return projectMapper.deleteProjectByIds(ids);
    }

    /**
     * 删除项目信息
     * 
     * @param id 项目主键
     * @return 结果
     */
    @Override
    public int deleteProjectById(Long id)
    {
        return projectMapper.deleteProjectById(id);
    }

    /**
     * 单条条件查询项目
     *
     * @param project 项目
     * @return 项目条目
     */
    @Override
    public Project selectProject(Project project) {
        return projectMapper.selectProject(project);
    }

    /**
     * 条件查询项目数量
     *
     * @param project 项目
     * @return 项目数量
     */
    @Override
    public Long selectProjectCount(Project project) {
        return projectMapper.selectProjectCount(project);
    }

    /**
     * 条件查询项目是否存在
     *
     * @param project 项目
     * @return 项目是否存在
     */
    @Override
    public boolean selectProjectExists(Project project) {
        return projectMapper.selectProjectExists(project) > 0;
    }

    @Override
    public int switchDisable(Long id, String userName)
    {
        Project project = projectMapper.selectProjectById(id);
        ASSERT.NONNULL(project, "项目不存在");
        project.setUpdateBy(userName);
        project.setDisabled("0".equals(project.getDisabled()) ? "1" : "0");
        project.setUpdateTime(DateUtils.getNowDate());
        int r = projectMapper.updateProject(project);
        ProjectUtils.Set(project);
        return r;
    }
}
